class DataPipelineObject(dict):
    def __init__(self, name, id, **kwargs):
        self['name']   = name
        self['id']     = id
        self['fields'] = list()
        for k,v in kwargs.iteritems():
            setattr(self, k, v)

    def __setattr__(self, key, value):
        if key in ('id', 'name'):
            self[key] = value
        elif isinstance(value, DataPipelineObject):
            self['fields'].append({ 'key' : key, 'refValue' : value.id })
        elif isinstance(value, bool):
            self['fields'].append({ 'key' : key, 'stringValue' : str(value).lower() })
        elif isinstance(value, list):
            for item in value:
                self.__setattr__(key, item)
        else:
            self['fields'].append({ 'key' : key, 'stringValue' : str(value) })
        super(DataPipelineObject, self).__setattr__(key, value)

    def __getattr__(self, key):
        try:
            if key in ('id', 'name'):
                return self[key]
            return iter(x for x in self['fields'] if x['key'] == key).next()
        except StopIteration:
            super(DataPipelineObject, self).__getattr__(key)


class TypedDataPipelineObject(DataPipelineObject):
    def __init__(self, name, id, **kwargs):
        kwargs.setdefault('type', type(self).__name__)
        super(TypedDataPipelineObject, self).__init__(name, id, **kwargs)


class Schedule(TypedDataPipelineObject): pass



from .base import DataPipelineObject, TypedDataPipelineObject, Schedule
from .actions import SnsAlarm, Terminate
from .activities import CopyActivity, EmrActivity, HiveActivity, HiveCopyActivity, \
    PigActivity, RedshiftCopyActivity, ShellCommandActivity, SqlActivity, \
    ShellCommand
from .data_formats import CSV, Custom, DynamoDBDataFormat, DynamoDBExportDataFormat, \
    RegEx, TSV
from .data_nodes import DynamoDBDataNode, MySqlDataNode, RedshiftDataNode, S3DataNode, \
    SqlDataNode
from .databases import JdbcDatabase, RdsDatabase, RedshiftDatabase
from .pipeline import Pipeline
from .preconditons import DynamoDBDataExists, DynamoDBTableExists, Exists, S3KeyExists, \
    S3PrefixNotEmpty, ShellCommandPrecondition
from .resources import Ec2Resource, EmrCluster

# Constants

class _StringConstant(dict):
    def __init__(self, *args):
        for k in args:
            self[k] = k

    def __getattr__(self, key):
        try:
            return self[key]
        except KeyError:
            super(DataPipelineObject, self).__getattr__(key)


failureAndRerunMode     = _StringConstant('CASCADE', 'NONE')
scheduleType            = _StringConstant('timeseries', 'cron')
startAt                 = _StringConstant('FIRST_ACTIVATION_DATE_TIME')
actionOnTaskFailure     = _StringConstant('continue', 'terminate')
actionOnResourceFailure = _StringConstant('retryAll', 'retryNone')



import boto.datapipeline

class Pipeline(object):
    def __init__(self, name, unique_id, desc=None, region='us-east-1', pipeline_id=None):
        self.name        = name
        self.unique_id   = unique_id
        self.desc        = desc
        self.region_name = region
        self.pipeline_id = pipeline_id
        self.objects     = list()

    @property
    def region(self):
        try:
            return self._region
        except AttributeError:
            self._region = self.connect()
            return self._region

    def connect(self, **kwargs):
        """ Connect to AWS region. """
        return boto.datapipeline.connect_to_region(self.region_name, **kwargs)

    def activate(self, pipeline_id=None):
        """ Activate pipeline. """
        pipeline_id = pipeline_id or self.pipeline_id
        return self.region.activate_pipeline(pipeline_id)

    def update(self, pipeline_id=None):
        """ Update pipeline definition with self.objects. """
        pipeline_id = pipeline_id or self.pipeline_id
        return self.region.put_pipeline_definition(self.objects, pipeline_id)

    def create(self):
        """ Create pipeline and set pipeline_id. """
        pipeline = self.region.create_pipeline(self.name, self.unique_id, self.desc)
        self.pipeline_id = pipeline['pipelineId']
        self.update()
        return pipeline

    def add(self, *objects):
        """ Add pipeline objects to pipeline. """
        self.objects += objects



from . import base

class DynamoDBDataExists(base.TypedDataPipelineObject): pass


class DynamoDBTableExists(base.TypedDataPipelineObject): pass


class Exists(base.TypedDataPipelineObject): pass


class S3KeyExists(base.TypedDataPipelineObject): pass


class S3PrefixNotEmpty(base.TypedDataPipelineObject): pass


class ShellCommandPrecondition(base.TypedDataPipelineObject): pass



from . import base

class CSV(base.TypedDataPipelineObject): pass


class Custom(base.TypedDataPipelineObject): pass


class DynamoDBDataFormat(base.TypedDataPipelineObject): pass


class DynamoDBExportDataFormat(base.TypedDataPipelineObject): pass


class RegEx(base.TypedDataPipelineObject): pass


class TSV(base.TypedDataPipelineObject): pass



from . import base

class DynamoDBDataNode(base.TypedDataPipelineObject): pass


class MySqlDataNode(base.TypedDataPipelineObject): pass


class RedshiftDataNode(base.TypedDataPipelineObject): pass


class S3DataNode(base.TypedDataPipelineObject): pass


class SqlDataNode(base.TypedDataPipelineObject): pass